/*******************************************************************************

		SC02_Boot.c	SC02 Boot mode class Ver.1.1

											2015.7.28 M.Utsumi@ARCDEVICE

*******************************************************************************/

#include <Windows.h>
#include<stdio.h>
#include "SC02_Boot.h"



SC02_Boot::SC02_Boot()
{
	ftHandle = NULL;
}

SC02_Boot::~SC02_Boot()
{
	if( ftHandle ) FT_Close( ftHandle );
}


BOOL SC02_Boot::Open( BYTE bTargetID, BYTE bBootMode, BYTE bChildID )
{
	DWORD dw, dwNumDevs;
	BYTE bRet;
	
	Error [0] ='\0'; 

	ftStatus = FT_ListDevices( &dwNumDevs, NULL, FT_LIST_NUMBER_ONLY );
	if (ftStatus != FT_OK){
		sprintf_s( Error , "FT_ListDevices failed\n");
		return( FALSE ); 
	}
	if( dwNumDevs==0 ){
		sprintf_s(Error , "Found no FT device\n");
		return( FALSE ); 
	}

	for(dw=0;dw<dwNumDevs;dw++){
		ftStatus = FT_Open( dw, &ftHandle);
		if (ftStatus != FT_OK){
			sprintf_s( Error , "FT_Open failed\n");
			return( FALSE ); 
		}

		//printf("Reset ");
		Reset();

		// Zbg21G[ɂȂ̂ł蒼D
		if(!putb( bTargetID )){
			Close();
			ftStatus = FT_Open( dw, &ftHandle);
			if (ftStatus != FT_OK){
				sprintf_s( Error , "FT_Open failed\n");
				return( FALSE ); 
			}

			Reset();

			if(!putb( bTargetID )) return( FALSE );
			
		}
		if(!putb( bBootMode )) return( FALSE );
		if(!getb( &bRet )) return( FALSE );

		if( bRet == bBootMode ){
			//printf("Mode %d", bBootMode);
			SetTimeout( 5000 );	// Timeout 5 sec
			printf("OK\n");

			if( bBootMode == CM_WRITER_MODE ){
				printf("Open child device:ID %d ", bChildID );
				if(!putb( bChildID )) return( FALSE );
				if(!getb( &bRet )) return( FALSE );
				if( bRet == 0 ){
					Close(); // Close SC02 
					sprintf_s(Error , "not existing.", bRet);
					return( FALSE );
				}else if( bRet != CM_PROGRAMING_MODE ){
					Close(); // Close SC02 
					sprintf_s(Error , "return value %d", bRet);
					return( FALSE );
				}
			}

			return( TRUE );
		}

		Close(); 
	}
	sprintf_s(Error , "Failed: There is no ID %d device\n", bTargetID );
	return( FALSE );
}

void SC02_Boot::Close( void )
{
	FT_Close( ftHandle );
	ftHandle = NULL;
}

BOOL SC02_Boot::Reset( void )
{
	SetTimeout( 5000 );	// Timeout 5 sec
	ftStatus = FT_ResetPort( ftHandle ); // Reset port
	if (ftStatus != FT_OK){
		sprintf_s( Error , "FT_ResetPort failed\n");
		return( FALSE ); 
	}
	Sleep(300);
	return( TRUE );

}

BOOL SC02_Boot::SetTimeout( WORD wTime_ms  )
{
	ftStatus = FT_SetTimeouts( ftHandle, wTime_ms, wTime_ms );
	if (ftStatus != FT_OK){
		sprintf_s( Error , "FT_SetTimeouts failed\n");
		return( FALSE ); 
	}
	return( TRUE );
}

BOOL SC02_Boot::getb( BYTE* pb )
{
	ftStatus = FT_Read( ftHandle, pb, 1, &dwBytesReceived );
	if( ftStatus!=FT_OK ){
		sprintf_s( Error , "getb ftStatus %d",ftStatus );
		Timeout = FALSE;
		*pb = 0;
		return( FALSE );
	}else{
		if( dwBytesReceived ==0 ){
            sprintf_s( Error , "Time-out in getb\n");
			Timeout = TRUE;
			*pb = 0;
			return( FALSE );
		}else{
			Timeout = FALSE;
		}
	}
	return( TRUE );
}

BOOL SC02_Boot::getw( WORD* pw )
{
	ftStatus = FT_Read( ftHandle, pw, 2, &dwBytesReceived );
	if( ftStatus!=FT_OK ){
		sprintf_s( Error , "getw ftStatus %d", ftStatus);
		Timeout = FALSE;
		*pw=0;
	}else{
		if( dwBytesReceived !=2 ){
			sprintf_s( Error , "Time-out in getw\n");
			Timeout = TRUE;
			*pw = 0;
		}else{
			Timeout = FALSE;
		}
	}
	
	return( TRUE );
}


BOOL SC02_Boot::Read( void* pBuf, WORD wSize )
{

	ftStatus = FT_Read( ftHandle, pBuf, wSize, &dwBytesReceived );
	if( ftStatus!=FT_OK ){
		sprintf_s( Error , "Read ftStatus %d", ftStatus );
		return( FALSE );
	}
	if( dwBytesReceived!=wSize ){
		sprintf_s( Error , "Time-out in Read\n");
		Timeout = TRUE;
		return( FALSE );
	}
	Timeout = FALSE;
	return( TRUE );
}

BOOL SC02_Boot::putb( BYTE b )
{
	ftStatus = FT_Write( ftHandle, &b, 1, &dwBytesWritten );
	if( ftStatus!=FT_OK ){
		sprintf_s( Error , "putb ftStatus %d", ftStatus );
		return(FALSE); 
	}else if( dwBytesWritten != 1){
		sprintf_s( Error , "putb BytesWritten miss match." );
		return(FALSE); 
	}

	return(TRUE); 
}

BOOL SC02_Boot::putw( WORD w )
{
	ftStatus = FT_Write( ftHandle, &w, 2, &dwBytesWritten );
	if( ftStatus!=FT_OK ){
		sprintf_s( Error , "putw ftStatus %d", ftStatus);
		return(FALSE); 
	}else if( dwBytesWritten != 2){
		sprintf_s( Error , "putw BytesWritten miss match." );
		return(FALSE); 
	}

	return(TRUE); 
}

BOOL SC02_Boot::Write( void* pBuf, WORD wSize )
{

	ftStatus = FT_Write( ftHandle, pBuf, wSize, &dwBytesWritten );
	if( ftStatus!=FT_OK ){
		sprintf_s( Error , "Write ftStatus %d", ftStatus );
		return( FALSE );
	}else if( dwBytesWritten != wSize){
		sprintf_s( Error , "Write BytesWritten miss match." );
		return(FALSE); 
	}

	return(TRUE); 
}

